package hage;

import kha.Color;
import kha.FontStyle;
import kha.Framebuffer;
import kha.Game;
import kha.graphics4.Graphics2;
import kha.Image;
import kha.Loader;
import kha.Scaler;
import kha.Scheduler;
import kha.ScreenRotation;

#if js
import js.Browser;
#elseif flash
import flash.events.Event;
import flash.Lib;
#end

/**
 * ...
 * @author JanGamesCo
 */
class HGEGame extends Game
{
	/***Public vars***/
	/*Game basic*/
	//Container to store all the game objects
	public var container:HGEBasic;
	//Game ID, should always be 0
	public var ID:Int;
	//Total game time
	public var gameTime:Float;
	
	/***Private vars***/
	private var wasPaused:Bool;	
	
	/***Public functions***/
	/*Init*/
	public function new(Name:String):Void
	{
		super(Name);
		
		//Set game reference
		HGE.game = this;
		
		//Get game ID
		ID = HGE.getID();
		
		//Init game container, all the game stuff is added to this
		container = new HGEBasic();
		
		//Init game components
		HGE.input = new HGEInput();
		
		//Init last time for delta
		HGE.lastTime = Scheduler.time();
		
		//Clear input on focus lost
		#if js
		var canvas:Dynamic = Browser.document.getElementById("khanvas");
		canvas.onblur = function() {
			pause();
		}
		canvas.onfocus = function() {
			resume();
		}
		#elseif flash
		Lib.current.stage.addEventListener(Event.DEACTIVATE, function(Ev:Event) { pause(); } );
		Lib.current.stage.addEventListener(Event.ACTIVATE, function(Ev:Event) { resume(); } );
		#end
	}

	/*Game add function*/
	public function add(Child:HGEBasic):Void { container.add(Child); }

	/*Remove a object function*/
	public function remove(Child:HGEBasic):Void { container.remove(Child); }
	
	//Pause event listener
	public function pause():Void {
		HGE.pause = true; HGE.input.clearInput();
	}
	
	//Unpause event listener
	public function resume():Void {
		HGE.pause = false;
		wasPaused = true;
	}
	
	/*Init game*/
	override public function init() {
		super.init();
		
		//Set width and height
		HGE.width = width;
		HGE.height = height;
		
		//Init backbuffer
		HGE.backbuffer = Image.createRenderTarget(HGE.width, HGE.height, null, false, 0);
	}
	
	private function initLevel() {		
	}
	
	/*Update event*/
	override public function update():Void
	{
		super.update();
		
		//Get delta time and lastTime
		getDelta();
		
		//Call update
		if (!HGE.pause) {
			//Update all childrens
			for(child in container.childrens) {
				if(child.exists) {
					child.update();
				}
			}
		}
		
		//Add to game time
		gameTime += HGE.delta;
		
		//Init debug mode
		#if debug
		if (HGE.input.keyPressed(HGEKeys.GRAVEACCENT)) { HGE.debug = !HGE.debug; }
		#end

		//Input update
		HGE.input.update();
	}
	
	//Render event
	override public function render(framebuffer:Framebuffer)
	{
		super.render(framebuffer);
		
		if (HGE.backbuffer == null) { return; }
		
		//Draw to backbuffer
		var g = HGE.backbuffer.g2;
		
		g.begin(true, 0);
		
		prerender();
		
		//Draw all childrens
		for(child in container.childrens) {
			if (child.visible) {
				child.draw();
			}
		}
		
		postrender();
		
		g.end();
		
		//Draw to framebuffer and draw debug data
		g = framebuffer.g2;
		g.begin();
		
		var t = g.transformation;
		
		Scaler.scale(HGE.backbuffer, framebuffer, ScreenRotation.RotationNone);
		
		g.transformation = t;
		
		//Draw debug data
		#if debug
		if(HGE.debug) {
			g.set_color(Color.White);
			g.font = Loader.the.loadFont("Arial", new FontStyle(false, false, false), 14);
			g.drawString("" + Math.floor(1 / HGE.delta), 10, 10);
		}
		#end
		
		g.end();
	}
	
	public function prerender()
	{
		return;
	}
	
	public function postrender()
	{
		return;
	}
	
	/***Private functions***/
	//Get delta time function
	private function getDelta():Void
	{
		HGE.delta = Scheduler.time() - HGE.lastTime;
		if (HGE.delta < 0 || wasPaused) {
			HGE.delta = 0;
			wasPaused = false;
		}
		HGE.lastTime = Scheduler.time();
	}

}